/*
 * Copyright (C) 2011 J. Coliz <maniacbug@ymail.com>
 * Inspired by cpixip.
 * See http://arduino.cc/forum/index.php/topic,54795.0.html
 * Modified for use on ELEC-A4010 Sähköpaja course.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 */

/**
 * Kanavaskanneri nRF24L01+ radiopiireille
 *
 * Tämä esimerkki mittaa häiriöitä radiopiirin mahdollisilta
 * kanavilta.  Tätä voidaan käyttää valitsemaan käyttämätön
 * radiokanava sovelluksillesi.
 *
 * Radiopiiri ei osaa suoraan mitata tehoa, vaan
 * mittaus käyttää yksinkertaisempaa havaitsinta joka antaa
 * merkinnän mikäli kanavalta havaitaan kantoaalto, eli
 * aktiivinen lähetys.  Toinen vaihtoehto on mitata havaitaanko
 * kanavalla yli -65dBm tehoa.
 *
 * Mittaus toistetaan vuoron perään kaikille 128 kanavalle
 * 100 kertaa.  Mittauksista kootaan tilasto, joka tulostetaan
 * PC:lle.
 *
 */

#include <SPI.h>
#include "nRF24L01.h"
#include "RF24.h"
#include "printf.h"

//
// Rauta-asetukset
//

// Kytke nRF24L01+ radio SPI väylään ja pinnit 9=CE ja 10=CSN.
RF24 radio(9,10);

//
// Radiokanavatiedot
//

const uint8_t num_channels = 128;  // Montako kanava
uint8_t values[num_channels];      // Häiriölaskurit kullekin kanavalle

const boolean measure_carrier = true; // Mitataanko kantoaaltoa vai tehoa

//
// Alustus
//

void setup(void)
{
  // Tulosta esittely
  Serial.begin(57600);
  printf_begin();
  printf("\n\rRF24/examples/scanner/\n\r");

  if (measure_carrier)  // Mitataan kantoaallon havaitsemista
    printf("Measuring carrier detection\n\r");
  else  // Mitataan vastaanotettua tehoa
    printf("Measuring received power density\n\r");

  // Alusta radiopiiri
  radio.begin();
  radio.setAutoAck(false);  // Ei kuittauksia

  // Laitetaan radio standby-tilaan
  radio.startListening();  // Käyntiin
  radio.stopListening();   // ja pois.

  // Tulosta kanavat heksana päällekkäisille riveille
  int i = 0;
  while ( i < num_channels ) {
    printf("%x",i>>4);  // Ensimmäinen heksadesimaali
    ++i;
  }
  printf("\n\r");
  i = 0;
  while ( i < num_channels ) {
    printf("%x",i&0xf);  // Toinen heksadesimaali
    ++i;
  }
  printf("\n\r");
}

//
// Pääsilmukka
//

const int num_reps = 100;  // Toistokertojen määrä

void loop(void)
{
  // Tyhjennä häiriölaskurit
  memset(values,0,sizeof(values));

  // Skannaa kaikki kanavat toistokertojen määrä
  int rep_counter = num_reps;
  while (rep_counter--) {
    int i = num_channels;
    while (i--) {
      // Valitse kanava
      radio.setChannel(i);

      // Kuuntele hetki
      radio.startListening();
      delayMicroseconds(128);
      radio.stopListening();

      // Havaittiinko signaali?
      if (( measure_carrier && radio.testCarrier() ) ||
          ( !measure_carrier && radio.testRPD() ))
        ++values[i];
    }
  }

  // Tulosta mittaustulokset yhdellä heksamerkillä per kanava
  int i = 0;
  while ( i < num_channels ) {
    printf("%x",min(0xf,values[i]&0xf)); // Tulostetaan vain alin heksadesimaali
    ++i;
  }
  printf("\n\r");
}

// vim:ai:cin:sts=2 sw=2 ft=cpp
