/*
 * Copyright (C) 2011 J. Coliz <maniacbug@ymail.com>
 * Modified for use on ELEC-A4010 Sähköpaja course.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 */

/**
 * Esimerkki nRF24L01+ tähtiverkosta
 *
 * Tässä esimerkissä laajennetaan pingpair-esimerkin kahden noodin
 * verkko kuuteen noodiin.  Käytetty radiopiiri tukee kuutta
 * vastaanottopuskuria, joten se soveltuu luonnostaan tilanteisiin
 * joissa piirillä on korkeintaan viisi keskustelukumppania.
 *
 * Kuten pingpair-esimerkissä, pinniä 7 käytetään roolivalinnassa.
 * Jos roolipinni on vedetty alas, noodi toimii kyselyjen
 * vastaanottajana (pong).  Jollei, noodi toimii kyselyjen
 * lähettäjänä (ping).  Lähettäjänoodit yksilöidään tarkemmin myös
 * EEPROM-muistiin tallennettavalla indeksillä.  Yksilöinnin tehdään
 * syöttämällä noodille merkin sarjaportilla PC:ltä.
 *
 * Perustoiminnallisuus on että ping-noodit lähettävät kyselyn joka
 * sisältää heidän kellonajan millisekunteina (millis()-kutsu) ja
 * odottavat että pong-noodi saa tämän viestin ja lähettää vastauksen
 * takaisin.  Vastauksen saadessaan ping-noodit tulostavat kauanko
 * vastauksen saamisessa kesti lähettämisestä.
 *
 */

#include <SPI.h>
#include <EEPROM.h>
#include "nRF24L01.h"
#include "RF24.h"
#include "printf.h"

//
// Rauta-asetukset
//

// Kytke nRF24L01+ radio SPI väylään ja pinnit 9=CE ja 10=CSN.
RF24 radio(9,10);

// Roolipinni valitsee tämän yksikön roolin.  Kytke maahan valitaksesi pong-noodi
// tai jätä avoimeksi valitaksesi ping-rooli.
const int role_pin = 7;

//
// Osoitteet
//

// Osoiteitta käytetään noodien väliseen viestintään.  Vastaanottoa varten
// tulee radiopiirille olla ohjelmoitu käytettävä osoite yhteen vastaanotto-
// putkeen.  Vain ping-noodit tarvitsevat erillisiä osoitteita ja pong-
// noodilla kuuluu olla kuunteluputki kutakin ping-noodia varten.  

// Radio pipe addresses for the nodes to communicate.  Only ping nodes need
// dedicated pipes in this topology.  Each ping node has a talking pipe
// that it will ping into, and a listening pipe that it will listen for
// the pong.  The pong node listens on all the ping node talking pipes
// and sends the pong back on the sending node's specific listening pipe.

const uint64_t talking_pipes[5] = { 0xF0F0F0F0D2LL, 0xF0F0F0F0C3LL, 0xF0F0F0F0B4LL, 0xF0F0F0F0A5LL, 0xF0F0F0F096LL };
const uint64_t listening_pipes[5] = { 0x3A3A3A3AD2LL, 0x3A3A3A3AC3LL, 0x3A3A3A3AB4LL, 0x3A3A3A3AA5LL, 0x3A3A3A3A96LL };

// EEPROM muistiosoite osoiteindeksin säilömiselle
const uint8_t address_at_eeprom_location = 0;

// Tämän noodin osoiteindeksi.  Kopioidaan EEPROMista käynnistyessä.
// Indeksi 0 on sovittu tarkoittamaan alustamatonta osoitetta.
// Indeksi 1 on pong-noodille, ja muut indeksit 2-6.
uint8_t node_address;



//
// Roolinhallinta
//
// Asettaa roolin.  Sama ohjelma ladataan kummallekin noodille.
// Tämä helpottaa ohjelmointia.  Kytkemällä roolipinni valitaan
// noodien toiminta.
//

// Käytetyt roolit
typedef enum { role_invalid = 0, role_ping_out = 1, role_pong_back } role_e;

// Ystävälliset nimet rooleille
const char* role_friendly_name[] = { "invalid", "Ping out", "Pong back"};

// Muuttuja joka säilöö noodin tämänhetkisen roolin
role_e role;

void setup(void)
{
  //
  // Roolin päättely
  //

  // Alusta roolipinni
  pinMode(role_pin, INPUT);
  digitalWrite(role_pin,HIGH);  // Aseta ylös
  delay(20); // Odota että tila vakiintuu

  // Lue roolipinnin arvo päätelläksesi noodin rooli
  if ( ! digitalRead(role_pin) )
    role = role_ping_out;
  else
    role = role_pong_back;

  //
  // Osoitteen valinta
  //

  if ( role == role_pong_back )
    node_address = 1;  // Pong-noodi, aina osoite 1
  else {
    // Ping-noodi, luetaan osoite EEPROMista

    // Lue EEPROM-muistipaikka
    uint8_t reading = EEPROM.read(address_at_eeprom_location);

    // Jos tulos on sallittu indeksi, käytetään sitä.
    if ( reading >= 2 && reading <= 6 )
      node_address = reading;
    // Muutoin osoite ei ole käyttökelpoinen ja roolimme on "invalid"
    // kunnes indeksi valitaan ja ohjelma käynnistetään uudelleen.
    else {
      node_address = 0;
      role = role_invalid;
    }
  }

  //
  // Tulosta aloitus
  //

  Serial.begin(57600);
  printf_begin();
  printf("\n\rRF24/examples/starping/\n\r");
  printf("ROLE: %s\n\r",role_friendly_name[role]);
  printf("ADDRESS: %i\n\r",node_address);

  //
  // Alusta radio
  //

  radio.begin();

  //
  // Alusta putket viestintää varten
  //

  // Pong-noodi tarvitsee kuunteluputket joka osoitteelle.
  if ( role == role_pong_back ) {
    radio.openReadingPipe(1,talking_pipes[0]);
    radio.openReadingPipe(2,talking_pipes[1]);
    radio.openReadingPipe(3,talking_pipes[2]);
    radio.openReadingPipe(4,talking_pipes[3]);
    radio.openReadingPipe(5,talking_pipes[4]);
    // Paluuviestit tehdään lähettäjän osoitteelle, joten
    // se voidaan alustaa vasta viestin saavuttua.
  }

  // Jokainen ping-noodi käyttää ominaista lähetysosoitettaan,
  // joten asetetaan sekä lähde- että kohdeosoite valmiiksi.
  if ( role == role_ping_out ) {
    // Lähetysosoite lähetysputkeen
    radio.openWritingPipe(talking_pipes[node_address-2]);
    // Kuunteluosoite vastaanottoputkeen
    radio.openReadingPipe(1,listening_pipes[node_address-2]);
  }

  //
  // Aloita kuuntelu
  //
  radio.startListening();

  //
  // Tulosta radiomodulin asetukset.
  //
  radio.printDetails();

  //
  // Jos rooli on "invalid", eli olemme ping-noodi ilman määriteltyä
  // osoiteindeksiä, tulosta varoitusviesti asiasta.
  //

  if ( role == role_invalid ) {
    printf("\n\r*** NO NODE ADDRESS ASSIGNED *** Send 1 through 6 to assign an address\n\r");
  }
}


//
// Pääsilmukka
//
void loop(void)
{
  //
  // Ping-noodi, lähettele jatkuvasti viestejä pong-noodille.
  //
  if (role == role_ping_out) {
    // Pong-noodit aloittavat lähetyksellä, joten lopetetaan kuuntelu.
    radio.stopListening();

    // Lue tämänhetkinen aika ja lähetä se radiolla.
    unsigned long time = millis();
    printf("Now sending %lu...",time);
    radio.write( &time, sizeof(unsigned long) );  // Odottaa kunnes viesti on läpi tai lähetys on epäonnistunut.

    // Vaihda kuunteluun jotta saadaan ping-noodin vastaus.
    radio.startListening();

    // Odota kunnes saadaan vastaus, tai on kulunut 250ms
    unsigned long started_waiting_at = millis();
    bool timeout = false;
    while ( ! radio.available() && ! timeout )
      if (millis() - started_waiting_at > 250 )
        timeout = true;

    // Kuvaile tulokset
    if ( timeout ) {
      // Ei saatu vastausta alle 250ms ajan.
      printf("Failed, response timed out.\n\r");
    } else {
      // Saatiin vastaus, lue siitä aikaleima ja vertaa nykyiseen.
      unsigned long got_time;
      radio.read( &got_time, sizeof(unsigned long) );  // Lue lähetysaika viestin sisällöstä.

      // Tulosta tulokset
      printf("Got response %lu, round-trip delay: %lu\n\r",got_time,millis()-got_time);
    }

    // Odotellaan 1s ennen uusintayritystä.
    delay(1000);
  }

  //
  // Pong-noodi, kuuntelee viestejä, tulostaa ne ja lähettää vastauksia.
  //

  if ( role == role_pong_back ) {
    // Testataan onko viestejä saatavilla, tallentaen putken indeksi pipe_num muuttujaan.
    uint8_t pipe_num;
    if ( radio.available(&pipe_num) ) {
      // On, joten luetaan kunnes ollaan saatu kaikki luettua.
      // Idea tässä on että jos useampi ping-viesti on jonossa,
      // kerätään vanhemmat pois ja vastataan vain uusimpaan.
      unsigned long got_time;  // <- puskuri johon aikaleima tallennetaan.
      bool done = false;
      while (!done) {
        // Lue viestin sisältö ja katso oliko tämä viimeinen viesti (done == true jos on).
        done = radio.read( &got_time, sizeof(unsigned long) );

        // Tulosta viestin tiedot.
        printf("Got payload %lu from node %i...",got_time,pipe_num+1);
      }

      // Viesti vastaanotettu, joten lopetetaan kuuntelu vastaamista varten.
      radio.stopListening();

      // Vaihdetaan lähetysputken osoitteeksi viestin lähettänyt noodi.
      radio.openWritingPipe(listening_pipes[pipe_num-1]);

      // Poimitaan kaksi alinta tavua osoitteesta putken tunnisteeksi.
      uint16_t pipe_id = listening_pipes[pipe_num-1] & 0xffff;

      // Lähetä vastausviesti.
      radio.write( &got_time, sizeof(unsigned long) );
      printf("Sent response to %04x.\n\r",pipe_id);

      // Jatketaan kuuntelua.
      radio.startListening();
    }
  }

  //
  // Luetaan sarjaportista halutaanko ping-noodin osoitteen indeksiä vaihtaa
  //
  if (Serial.available()) {
    // Luetaann merkki sarjaportista.
    char c = Serial.read();
    // Jos merkki on numero väliltä 1-6..
    if ( c >= '1' && c <= '6' ) {
      // Tallennetaan se EEPROMiin meidän osoitteeksi.
      EEPROM.write(address_at_eeprom_location,c-'0');

      // Jotta osoite luetaan käyttöön, piiri pitää resetoida..
      // (helpoin tapa hoitaa tämä, ettei tarvi yrittää uudelleen-
      //  alustaa kesken toiminnan..)
      printf("\n\rManually reset address to: %c\n\rPress RESET to continue!",c);
      while(1) ;  // Jäädään tähän jumiin odottamaan resettiä.
    }
  }
}
// vim:ai:ci sts=2 sw=2 ft=cpp
